==> comedi-0.7.31.tgz: comedi-0.7.31/module/drivers.c <==
/*
    module/drivers.c
    functions for manipulating drivers

    COMEDI - Linux Control and Measurement Device Interface
    Copyright (C) 1997-8 David A. Schleef <ds@stm.lbl.gov>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <comedi_module.h>

#include <linux/module.h>
#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/fcntl.h>
#include <linux/delay.h>
#include <linux/ioport.h>
#include <linux/mm.h>
#include <linux/malloc.h>
#include <asm/io.h>

static void postconfig(comedi_device *dev);

comedi_driver *comedi_drivers;

int comedi_modprobe(kdev_t minor)
{
	return -EINVAL;
}

int comedi_device_detach(comedi_device *dev)
{
	int i;
	comedi_subdevice *s;

	if(!dev->attached)
		return 0;

	dev->attached=0;

	for(i=0;i<dev->n_subdevices;i++){
		s=dev->subdevices+i;
		if(s->prealloc_buf)
			vfree(s->prealloc_buf);
	}

	if(dev->driver){
		dev->driver->detach(dev);
	}else{
		if(dev->rem)dev->rem(dev);
	}

	if(dev->subdevices)kfree(dev->subdevices);
	if(dev->private)kfree(dev->private);

	return 0;
}

int comedi_device_attach(comedi_device *dev,comedi_devconfig *it)
{
	comedi_driver *driv;
	int i,ret;

	if(dev->attached)
		return -EBUSY;

	for(driv=comedi_drivers;driv;driv=driv->next){
		if(driv->recognize){
			i=driv->recognize(it->board_name);
			if(i<0)continue;
		}else{
			if(strcmp(driv->driver_name,it->board_name))
				continue;
		}
		ret=driv->attach(dev,it);
		if(ret<0)return ret;
		goto attached;
	}

	for(i=0;i<comedi_maxinits;i++){
		ret=comedi_devinits[i](dev,it);
		if(ret<0)return ret;
		if(ret)goto attached;
	}
	return -EIO;

attached:
	init_waitqueue_head(&dev->wait);

	/* do a little post-config cleanup */
	postconfig(dev);

	dev->attached=1;
	dev->driver=driv;

	return 0;
}

int comedi_driver_register(comedi_driver *driver)
{
	driver->next=comedi_drivers;
	comedi_drivers=driver;

	return 0;
}

int comedi_driver_unregister(comedi_driver *driver)
{
	comedi_driver *prev;

	if(comedi_drivers==driver){
		comedi_drivers=driver->next;
		return 0;
	}

	for(prev=comedi_drivers;prev->next;prev=prev->next){
		if(prev->next==driver){
			prev->next=driver->next;
			return 0;
		}
	}
	return -EINVAL;
}


static void postconfig(comedi_device *dev)
{
	int i;
	comedi_subdevice *s;

	for(i=0;i<dev->n_subdevices;i++){
		s=dev->subdevices+i;

		if(s->type==COMEDI_SUBD_UNUSED)
			continue;

		if(s->len_chanlist==0)
			s->len_chanlist=1;

		/* XXX */
		if(s->trig[1] || s->trig[2] || s->trig[3] ||s->trig[4]){
			s->prealloc_bufsz=1024*128;
		}else{
			s->prealloc_bufsz=0;
		}

		if(s->prealloc_bufsz){
			/* XXX */
			s->prealloc_buf=vmalloc(s->prealloc_bufsz*sizeof(sampl_t));
			if(!s->prealloc_buf){
				printk("ENOMEM\n");
			}
		}

		if(!s->range_type && !s->range_type_list)
			s->range_type=RANGE_unknown;
	}

}


extern comedi_driver driver_dt282x;

void init_drivers(void)
{
#ifdef CONFIG_DT282x
	comedi_driver_register(&driver_dt282x);
#endif
}


/* old devinit initialization */

comedi_devinit dummy_init,
	dt2811_init,
	dt2801_init,
	dt2814_init,
	dt2817_init,
	dt282x_init,
	atmio_E_init,
	pcimio_E_init,
	nidio_init,
	atmio16f_init,
	ssc31_init,
	dev_8255_init,
	das08_init,
	pcl711_init,
	pcl725_init,
	pcl726_init,
	rti800_init,
	rti802_init,
	rti860_init,
	parport_init,
        acl8112_init,
        das1600_init,
        das6402_init,
	das08jr_init,
        multiq3_init,
        dt2815_init,
        dt3000_init,
	das16_init,
	xxx_init,
	dds_init;

comedi_devinit *comedi_devinits[]={
/* dummy driver */
	dummy_init,
#ifdef CONFIG_DT2801
/* Data Translation DT2801 board */
	dt2801_init,
#endif
#ifdef CONFIG_DT2811
/* Data Translation DT2811 board */
	dt2811_init,
#endif
#ifdef CONFIG_DT2814
/* Data Translation DT2814 board */
	dt2814_init,
#endif
#ifdef CONFIG_DT2817
/* Data Translation DT2817 board */
	dt2817_init,
#endif
#ifdef CONFIG_DT3000
/* Data Translation DT3000 series boards */
	dt3000_init,
#endif
#ifdef CONFIG_8255
/* generic 8255 */
	dev_8255_init,
#endif
#ifdef CONFIG_ATMIO_E
/* National Intruments NI-DAQ AT-MIO E series */
	atmio_E_init,
#endif
#ifdef CONFIG_PCIMIO_E
/* National Intruments NI-DAQ PCI-MIO E series */
	pcimio_E_init,
#endif
#ifdef CONFIG_NI_F
/* National Intruments NI-DAQ AT-MIO, Am9513 based */
	ni_F_init,
#endif
#ifdef CONFIG_NI_PCIDIO
/* National Intruments PCI-DIO-96 */
	nidio_init,
#endif
#ifdef CONFIG_SSC31
/* Spectrum Signal Processing, Inc. PC/C31 Real Time Application board */
	ssc31_init,
#endif
#ifdef CONFIG_DAS08
/* DAS-08 driver for Keithley Metrabyte, Computer Boards */
	das08_init,
#endif
#ifdef CONFIG_PCL711
/* PCL-711 PC-LabCard */
	pcl711_init,
#endif
#ifdef CONFIG_PCL725
/* pcl 725 and compatibles */
	pcl725_init,
#endif
#ifdef CONFIG_PCL726
/* PCL-726 PC-LabCard */
	pcl726_init,
#endif
#ifdef CONFIG_RTI800
/* Analog Devices RTI800/815 */
	rti800_init,
#endif
#ifdef CONFIG_RTI802
/* Analog Devices RTI802 */
	rti802_init,
#endif
#ifdef CONFIG_RTI860
	rti860_init,
#endif
#ifdef CONFIG_COMEDI_PARPORT
/* Parallel port */
	parport_init,
#endif
#ifdef CONFIG_DAS08JR
/* DAS-08jr driver for Keithley Metrabyte, Computer Boards */
	das08jr_init,
#endif
#ifdef CONFIG_DAS1600
/* DAS-1600 driver for Keithley Metrabyte, Computer Boards */
	das1600_init,
#endif
#ifdef CONFIG_DAS6402
/* DAS-6402 driver for Keithley Metrabyte, Computer Boards */
	das6402_init,
#endif
#ifdef CONFIG_MULTIQ3
/* MultiQ-3 driver for Quanser Consulting MultiQ-3 board. */
	multiq3_init,
#endif
#ifdef CONFIG_DT2815
/* Data Translation DT2815 series board. */
       dt2815_init,
#endif
#ifdef CONFIG_DAS16
/* DAS-16 driver for Keithley Metrabyte, Computer Boards */
	das16_init,
#endif
#ifdef CONFIG_DAS16
/* National Instruments PCI-DIO-96 */
	nidio96_init,
#endif
#ifdef CONFIG_VD_XXX
/* virtual driver XXX */
	xxx_init,
#endif
#ifdef CONFIG_VD_DDS
/* virtual driver DDS */
	dds_init,
#endif
	NULL
};

const int comedi_maxinits=sizeof(comedi_devinits)/sizeof(comedi_devinit *)-1;

==> comedi-0.7.32.tgz: comedi-0.7.32/module/drivers.c <==
/*
    module/drivers.c
    functions for manipulating drivers

    COMEDI - Linux Control and Measurement Device Interface
    Copyright (C) 1997-8 David A. Schleef <ds@stm.lbl.gov>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <comedi_module.h>
#include <kvmem.h>

#include <linux/module.h>
#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/fcntl.h>
#include <linux/delay.h>
#include <linux/ioport.h>
#include <linux/mm.h>
#include <linux/malloc.h>
#include <asm/io.h>

static void postconfig(comedi_device *dev);

comedi_driver *comedi_drivers;

int comedi_modprobe(kdev_t minor)
{
	return -EINVAL;
}

int comedi_device_detach(comedi_device *dev)
{
	int i;
	comedi_subdevice *s;

	if(!dev->attached)
		return 0;

	dev->attached=0;

	for(i=0;i<dev->n_subdevices;i++){
		s=dev->subdevices+i;
		if(s->prealloc_buf)
			vfree(s->prealloc_buf);
	}

	if(dev->driver){
		dev->driver->detach(dev);
	}else{
		if(dev->rem)dev->rem(dev);
	}

	if(dev->subdevices)kfree(dev->subdevices);
	if(dev->private)kfree(dev->private);

	return 0;
}

int comedi_device_attach(comedi_device *dev,comedi_devconfig *it)
{
	comedi_driver *driv;
	int i,ret;

	if(dev->attached)
		return -EBUSY;

	for(driv=comedi_drivers;driv;driv=driv->next){
		if(driv->recognize){
			i=driv->recognize(it->board_name);
			if(i<0)continue;
		}else{
			if(strcmp(driv->driver_name,it->board_name))
				continue;
		}
		ret=driv->attach(dev,it);
		if(ret<0)return ret;
		goto attached;
	}

	for(i=0;i<comedi_maxinits;i++){
		ret=comedi_devinits[i](dev,it);
		if(ret<0)return ret;
		if(ret)goto attached;
	}
	return -EIO;

attached:
	init_waitqueue_head(&dev->wait);

	/* do a little post-config cleanup */
	postconfig(dev);

	dev->attached=1;
	dev->driver=driv;

	return 0;
}

int comedi_driver_register(comedi_driver *driver)
{
	driver->next=comedi_drivers;
	comedi_drivers=driver;

	return 0;
}

int comedi_driver_unregister(comedi_driver *driver)
{
	comedi_driver *prev;

	if(comedi_drivers==driver){
		comedi_drivers=driver->next;
		return 0;
	}

	for(prev=comedi_drivers;prev->next;prev=prev->next){
		if(prev->next==driver){
			prev->next=driver->next;
			return 0;
		}
	}
	return -EINVAL;
}


static void postconfig(comedi_device *dev)
{
	int i;
	comedi_subdevice *s;

	for(i=0;i<dev->n_subdevices;i++){
		s=dev->subdevices+i;

		if(s->type==COMEDI_SUBD_UNUSED)
			continue;

		if(s->len_chanlist==0)
			s->len_chanlist=1;

		/* XXX */
		if(s->trig[1] || s->trig[2] || s->trig[3] ||s->trig[4]){
			s->prealloc_bufsz=1024*128;
		}else{
			s->prealloc_bufsz=0;
		}

		if(s->prealloc_bufsz){
			/* XXX */
			s->prealloc_buf=rvmalloc(s->prealloc_bufsz*sizeof(sampl_t));
			if(!s->prealloc_buf){
				printk("ENOMEM\n");
			}
		}

		if(!s->range_type && !s->range_type_list)
			s->range_type=RANGE_unknown;
	}

}


extern comedi_driver driver_dt282x;

void init_drivers(void)
{
#ifdef CONFIG_DT282x
	comedi_driver_register(&driver_dt282x);
#endif
}


/* old devinit initialization */

comedi_devinit dummy_init,
	dt2811_init,
	dt2801_init,
	dt2814_init,
	dt2817_init,
	dt282x_init,
	atmio_E_init,
	pcimio_E_init,
	nidio_init,
	atmio16f_init,
	ssc31_init,
	dev_8255_init,
	das08_init,
	pcl711_init,
	pcl725_init,
	pcl726_init,
	rti800_init,
	rti802_init,
	rti860_init,
	parport_init,
        acl8112_init,
        das1600_init,
        das6402_init,
	das08jr_init,
        multiq3_init,
        dt2815_init,
        dt3000_init,
	das16_init,
	xxx_init,
	dds_init;

comedi_devinit *comedi_devinits[]={
/* dummy driver */
	dummy_init,
#ifdef CONFIG_DT2801
/* Data Translation DT2801 board */
	dt2801_init,
#endif
#ifdef CONFIG_DT2811
/* Data Translation DT2811 board */
	dt2811_init,
#endif
#ifdef CONFIG_DT2814
/* Data Translation DT2814 board */
	dt2814_init,
#endif
#ifdef CONFIG_DT2817
/* Data Translation DT2817 board */
	dt2817_init,
#endif
#ifdef CONFIG_DT3000
/* Data Translation DT3000 series boards */
	dt3000_init,
#endif
#ifdef CONFIG_8255
/* generic 8255 */
	dev_8255_init,
#endif
#ifdef CONFIG_ATMIO_E
/* National Intruments NI-DAQ AT-MIO E series */
	atmio_E_init,
#endif
#ifdef CONFIG_PCIMIO_E
/* National Intruments NI-DAQ PCI-MIO E series */
	pcimio_E_init,
#endif
#ifdef CONFIG_NI_F
/* National Intruments NI-DAQ AT-MIO, Am9513 based */
	ni_F_init,
#endif
#ifdef CONFIG_NI_PCIDIO
/* National Intruments PCI-DIO-96 */
	nidio_init,
#endif
#ifdef CONFIG_SSC31
/* Spectrum Signal Processing, Inc. PC/C31 Real Time Application board */
	ssc31_init,
#endif
#ifdef CONFIG_DAS08
/* DAS-08 driver for Keithley Metrabyte, Computer Boards */
	das08_init,
#endif
#ifdef CONFIG_PCL711
/* PCL-711 PC-LabCard */
	pcl711_init,
#endif
#ifdef CONFIG_PCL725
/* pcl 725 and compatibles */
	pcl725_init,
#endif
#ifdef CONFIG_PCL726
/* PCL-726 PC-LabCard */
	pcl726_init,
#endif
#ifdef CONFIG_RTI800
/* Analog Devices RTI800/815 */
	rti800_init,
#endif
#ifdef CONFIG_RTI802
/* Analog Devices RTI802 */
	rti802_init,
#endif
#ifdef CONFIG_RTI860
	rti860_init,
#endif
#ifdef CONFIG_COMEDI_PARPORT
/* Parallel port */
	parport_init,
#endif
#ifdef CONFIG_DAS08JR
/* DAS-08jr driver for Keithley Metrabyte, Computer Boards */
	das08jr_init,
#endif
#ifdef CONFIG_DAS1600
/* DAS-1600 driver for Keithley Metrabyte, Computer Boards */
	das1600_init,
#endif
#ifdef CONFIG_DAS6402
/* DAS-6402 driver for Keithley Metrabyte, Computer Boards */
	das6402_init,
#endif
#ifdef CONFIG_MULTIQ3
/* MultiQ-3 driver for Quanser Consulting MultiQ-3 board. */
	multiq3_init,
#endif
#ifdef CONFIG_DT2815
/* Data Translation DT2815 series board. */
       dt2815_init,
#endif
#ifdef CONFIG_DAS16
/* DAS-16 driver for Keithley Metrabyte, Computer Boards */
	das16_init,
#endif
#ifdef CONFIG_DAS16
/* National Instruments PCI-DIO-96 */
	nidio96_init,
#endif
#ifdef CONFIG_VD_XXX
/* virtual driver XXX */
	xxx_init,
#endif
#ifdef CONFIG_VD_DDS
/* virtual driver DDS */
	dds_init,
#endif
	NULL
};

const int comedi_maxinits=sizeof(comedi_devinits)/sizeof(comedi_devinit *)-1;
